#!python3
# https://nagios-plugins.org/doc/guidelines.html

# Import required libs for sharepointhealth
from .azure_alerts_check import AzureAlerts
import argparse
import sys

# Return codes expected by Nagios
OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3

status = OK

# Return message
message = {
    'status': OK,
    'summary': 'Alerts summary',
    'perfdata': 'label1=0;;;; '  # 'label'=value[UOM];[warn];[crit];[min];[max] 
}

# For multiple perdata, ensure to add space after each perfdata
# message['perfdata'] = 'label1=x;;;; '
# message['perfdata'] += 'label2=x;;;; '

# Function to parse arguments
def parse_args(args):
    """
    Information extracted from: https://mkaz.com/2014/07/26/python-argparse-cookbook/
    https://docs.python.org/3/library/argparse.html
    :return: parse.parse_args(args) object
    You can use obj.option, example:
    options = parse_args(args)
    options.user # to read username
    """
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter, 
        description='Nagios plugin to check azure alerts')

    parser.add_argument('--tenant_id', dest='tenant_id', nargs='?', default=None, const=None, help='Header tenant ID of Azure \n')
    parser.add_argument('--client_id', dest='client_id', nargs='?', default=None, const=None, help='Header client_id \n')
    parser.add_argument('--client_secret', dest='client_secret', nargs='?', default=None, const=None, help='Header client_secret \n')
    parser.add_argument('--suscription_id', dest='suscription_id', nargs='?', default=None, const=None, help='Header suscription id of Azure \n')
    parser.add_argument('--time_range', dest='time_range' , nargs='?', default='1d', const=None, help='Time range for alerts - default=1d (accepted values: 1d, 1h, 30d, 7d) \n')
    parser.add_argument('-e', '--extra_args', dest='extra_args', nargs='?', default='', const=None, help='extra args to add to curl, see curl manpage  \n')

    if not args:
        raise SystemExit(parser.print_help())

    return parser.parse_args(args)

# Function to execute cli commands
def cli_execution(options):
    """
    : param: options: arguments from parse.parse_args(args) (see parse_args function)
    """
    #variables
    auth_args = '' 
    retrcode = OK
    
    if not options.tenant_id:
            sys.exit('param tenant_id is required  when using azure alerts check ')
    if not options.client_secret:
            sys.exit('param client_secret is required  when using azure alerts check ')
    if not options.client_id:
            sys.exit('param client_id is required  when using azure alerts check ')
    if not options.suscription_id:
            sys.exit('param suscription_id is required  when using azure alerts check ')

    # Get list of Azure alerts
    azure_alerts_obj = AzureAlerts(tenant_id = options.tenant_id, client_id = options.client_id,client_secret = options.client_secret,suscription_id = options.suscription_id, time_range = options.time_range)	

    azure_alerts_tuple = azure_alerts_obj.get_alerts()
    
    summary_alerts = azure_alerts_tuple[0]
    response_http_code = azure_alerts_tuple[1]   
    retrcode =  azure_alerts_tuple[2]   

    #if '200' not in response_http_code:        
    if response_http_code != 200:
        sys.exit('Error getting data from {} http_code != 200, http_code: {}'.format(response_http_code))

    def check(retrcode):
        if retrcode == 2:            
            message['summary'] = 'CRITICAL: ' + str(summary_alerts)
        else:
            global status 
            status = OK
            message['summary'] = 'OK: No alerts' 
        return status

    # Check logic starts here    
    message['status'] = check(retrcode)

    # Print the message
    print("{summary}".format(
        summary=message.get('summary'),
    ))

    # Exit with status code
    raise SystemExit(message['status'])

# Argument parser
# https://docs.python.org/3.5/library/argparse.html

def main():
    """
    Main function
    """
    # Get options with argparse
    options = parse_args(sys.argv[1:])
    # Execute program functions passing the options collected
    cli_execution(options)


if __name__ == "__main__":
    main()



##
## documentation:
import requests
import json
import jinja2
from requests import post

## Return codes expected by Nagios
OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3

status = OK

class AzureAlerts:
    def __init__(self, tenant_id, client_id, client_secret, suscription_id, time_range):  
        ## initialization
        ## At the beginning, the code will get the alerts information

        self.tenant_id = tenant_id
        self.client_id = client_id
        self.client_secret = client_secret
        self.suscription_id = suscription_id
        self.time_range = time_range

    def get_alerts(self):


        ## Vars
        api_version = '2019-03-01'
        alert_state = 'New'
        monitor_condition = 'Fired'
        authorization_token = self.get_token()

        text_to_print = ''
        counter = 0

        url_alerts = f'https://management.azure.com/subscriptions/{self.suscription_id}/providers/Microsoft.AlertsManagement/alerts?api-version={api_version}&alertState={alert_state}&timeRange={self.time_range}&monitorCondition={monitor_condition}'

        ## Headers
        headers_alerts = {
            'Authorization': "Bearer " + authorization_token,
            'Content-Type': 'application/json'
        }

        ## Retrive JSON
        response_alerts = requests.get(url=url_alerts, headers=headers_alerts)
        ## Checks if http error exit with traceback and error / https://stackoverflow.com/questions/61463224/when-to-use-raise-for-status-vs-status-code-testing
        ## https://www.codegrepper.com/code-examples/python/raise_for_status%28%29+requests
        try:
            response_alerts.raise_for_status()
            response_alerts_body = response_alerts.json()

            environment = jinja2.Environment()
            template = environment.from_string("""
            Name: {{ alert_name }}
            targetResourceName: {{ alert_targetResourceName }}
            targetResourceGroup: {{ alert_targetResourceGroup }}
            startDateTime: {{ alert_startDateTime }}
            Description: {{ alert_description }}

            """
            )            

            ## Start processing the json
            ## and generate the string to print
            ## Read the list of alerts from key 'value'
            for alert in response_alerts_body['value']:
                status = CRITICAL
                counter += 1
                alert_name = alert.get('name')
                alert_targetResourceName = alert.get('properties').get('essentials').get('targetResourceName')
                alert_targetResourceGroup = alert.get('properties').get('essentials').get('targetResourceGroup')
                alert_startDateTime = alert.get('properties').get('essentials').get('startDateTime')
                alert_description = alert.get('properties').get('essentials').get('description')

                ## Start concatenating the text template rendered with the values:
                text_to_print += template.render(
                    alert_name = alert_name,
                    alert_targetResourceName = alert_targetResourceName,
                    alert_targetResourceGroup = alert_targetResourceGroup,
                    alert_startDateTime = alert_startDateTime,
                    alert_description = alert_description
                )

                if counter == 10:
                    break

        except requests.exceptions.HTTPError as error:

            status = CRITICAL

            ## Example error output
            """"
            {"error":
                {"code":"AuthenticationFailedMissingToken","message":"Authentication failed. The 'Authorization' header is missing the access token."}
            }
            """                       
            
            ## Retrieve error message and code of request failed
            error_json = error.response.json()
            error_message = error_json['error']['message'] 
            error_code = error_json['error']['code'] 

            # Print the message
            print("Error request: " + error_message + "\nCode: " + error_code)
            
            ## Exit with message and code of error
            raise SystemExit(status)
            

        return text_to_print, response_alerts.status_code, status

    def get_token(self):

        ## Headers
        headers_token = {
            'Host': 'login.microsoftonline.com',
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        ## Body
        body = {
            'client_id': self.client_id,
            'scope': 'https://management.azure.com//.default',
            'client_secret': self.client_secret,
            'grant_type': 'client_credentials',
            'tenant': self.tenant_id
        }

        login_url = f'https://login.microsoftonline.com/{self.tenant_id}/oauth2/v2.0/token'

        ## Retrive JSON
        response_token = post(url=login_url, headers=headers_token, data=body)
        ## Checks if http error exit with traceback and error / https://stackoverflow.com/questions/61463224/when-to-use-raise-for-status-vs-status-code-testing
        ## https://www.codegrepper.com/code-examples/python/raise_for_status%28%29+requests
        try:
            response_token.raise_for_status() 
            response_alerts_body = response_token.json()
            ## Take token from json
            authorization_token = f"{response_alerts_body['access_token']}"
        except:
            authorization_token = ""

        return authorization_token
